import warnings

warnings.filterwarnings("ignore")
import whisper_timestamped as whisper
from whisper_timestamped.make_subtitles import write_srt, write_vtt, split_long_segments
warnings.filterwarnings("default")

import utils
import os
import typing


class Transcriber:
    def __init__(self, args: dict[str, typing.Any]) -> None:
        self.input = utils.fileExists(args.pop("input"))
        self.output = utils.dirExists(args.pop("output"))
        self.verbose = args.pop("verbose")
        self.format = args.pop("format")
        self.outputName = utils.outputFileName(args.pop("output_name"), self.format)
        self.maxChars = args.pop("max_chars")
        self.modelName = args.pop("model")

    def transcribe(self) -> None:
        utils.verbose(f"Loading model '{self.modelName}'...", self.verbose)
        model = whisper.load_model(self.modelName)

        utils.verbose(f"Starting transcription...", self.verbose)
        warnings.filterwarnings("ignore")
        result = whisper.transcribe(model=model, audio=self.input, verbose=False)
        warnings.filterwarnings("default")

        outputFilePath = os.path.join(self.output, self.outputName)

        if os.path.exists(outputFilePath):
            utils.overrideOutputFilePrompt()


        utils.verbose(f"Writing subtitle file...", self.verbose)
        with open(outputFilePath, "w", encoding="utf-8") as outputFile:
            if self.format == "srt":
                write_srt(self.getSubtitleData(result["segments"]), outputFile)
            else:
                write_vtt(self.getSubtitleData(result["segments"]), outputFile)
            
            outputFile.close()


    def getSubtitleData(self, segments: dict[str, typing.Any]) -> dict[str, typing.Any]:
        if self.maxChars == None:
            return segments
        else:
            return split_long_segments(segments, self.maxChars)

    @staticmethod
    def defaultModel() -> str:
        return "small"
    
    @staticmethod
    def availableModels() -> list[str]:
        return whisper.available_models()


import argparse
import pathlib
import os
from time import time

def checkPositive(value) -> int:
    ivalue = int(value)
    if ivalue <= 0:
        raise argparse.ArgumentTypeError(f"{value} is an invalid positive int value")
    return ivalue


def fileExists(path: str) -> str:
    if pathlib.Path(path).is_file():
        return path
    else:
        raise argparse.ArgumentError(None, "File does not exist")


def dirExists(path: str) -> str:
    if pathlib.Path(path).is_dir():
        return path
    else:
        raise argparse.ArgumentError(None, "Directory does not exist")


def generateName() -> str:
    return f"subtitle_{str(time()).replace('.', '')}"


def outputFileName(outputName: str, format: str) -> str:
    return f"{generateName() if outputName == None else outputName}.{format}"


def verbose(msg: str, verbose: bool):
    if (verbose):
        print(msg)


def overrideOutputFilePrompt(outputFilePath: str) -> None:
    override = input("\nOutput file already exists. Override [y/n]: ")

    if override != "y":
        print("Aborting file save...")
        exit()
    else:
        os.remove(outputFilePath)


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    version="0.1.1",
    name="astrotitles",
    packages=find_packages(),
    license="AGPL-3.0",
    author="Ben Webster",
    url="https://github.com/Astrotitles/cli",
    download_url="https://github.com/Astrotitles/cli.git",
    keywords=["AI", "Whisper AI", "Subtitles"],
    install_requires=[
        'whisper_timestamped',
    ],
    description="Automatically generate subtitles using the Astrotitles command line interface.",
    long_description=long_description,
    long_description_content_type="markdown",
    entry_points={
        'console_scripts': ['astrotitles=astrotitles.cli:main'],
    },
    include_package_data=True,
)


import argparse
import utils

from transcriber import Transcriber
from whisper.utils import str2bool

def cli():
    parser = argparse.ArgumentParser(
        prog="astrotitles",
        description="Generate a srt/vtt subtitle file from audio/video file",
        epilog="Thanks for using Astrotitles CLI!"
    )

    parser.add_argument("input", type=str, help="Input audio/video file to be transcribed")

    parser.add_argument("--output", type=str, default="./", help="Specify output directory for srt/vtt subtitle file")
    parser.add_argument("--output-name", type=str, help="Specify output file name (no extension)")
    parser.add_argument("--max-chars", type=utils.checkPositive, help="Specify the maximum number of characters allowed per subtitle segment")
    parser.add_argument("--format", type=str, default="srt", choices=["srt", "vtt"], metavar="FORMAT", help="Specify subtitle format (default: .srt)")
    parser.add_argument("--model", type=str, default=Transcriber.defaultModel(), choices=Transcriber.availableModels(), metavar="MODEL", help="Whisper AI model to use for transcribing")
    parser.add_argument("--verbose", type=str2bool, default=True, help="Print out progress of transcription process")

    args = parser.parse_args().__dict__

    transcriber = Transcriber(args)
    transcriber.transcribe()

if __name__ == "__main__":
    cli()